# Installation guide


```bash
conda create --name cwb
conda activate cwb
git clone https://gitlab.com/gwburst/public/library.git cwb_source
cd cwb_source
conda install -c conda-forge root=6.26.10
conda install -c conda-forge libframe=8.33 healpix_cxx=3.81
conda install -c conda-forge lal lalapps lalinspiral lalburst lalmetaio lalsimulation framel cfitsio
```

If you see the error `CMake 3.14.0 or higher is required.` install cmake in conda with
```bash
conda install -c anaconda cmake
```

Replace the following lines in `wat/CMakeLists.txt` with
```bash
# line 3: add ASM
    LANGUAGES CXX ASM

# line 52: change watasm_*.o to watasm.S
list(APPEND WAT_SRC watasm.S)

# line 65: change HEALPIX_LIBRARY to standard pkg-config variable HEALPIX_LIBRARIES
target_link_libraries(${PROJECT_NAME} ROOT::TreePlayer ROOT::Core ROOT::Graf ROOT::FFTW ROOT::Tree ROOT::Gpad ROOT::Physics ${HEALPIX_LIBRARIES})

# line 73 - end: add dylib support for macos
if(APPLE)
   set(lib_ext dylib)
else()
   set(lib_ext so)
endif()

add_custom_command(
    TARGET ${PROJECT_NAME} POST_BUILD
    COMMAND ln -sf wavelet-${XIFO}x.${lib_ext} wavelet.so
    COMMAND ln -sf wavelet.so libwavelet.so
    WORKING_DIRECTORY lib
)
install(FILES ${WAT_HDRS}
        TYPE INCLUDE)
install(FILES ${CMAKE_CURRENT_BINARY_DIR}/lib/wavelet-${XIFO}x.${lib_ext}
              ${CMAKE_CURRENT_BINARY_DIR}/lib/wavelet.so
              ${CMAKE_CURRENT_BINARY_DIR}/lib/libwavelet.so
              ${CMAKE_CURRENT_BINARY_DIR}/${CMAKE_SHARED_LIBRARY_PREFIX}${PROJECT_NAME}_rdict.pcm
        TYPE LIB)
```


Add flag in `tools/cwb/CMakeLists.txt` to prevent `Undefined Symbol: CWB_Plugin`
```bash
set(CMAKE_SHARED_LINKER_FLAGS "-undefined dynamic_lookup")
``` 


Run
```bash
export LIBRARY_PATH=$CONDA_PREFIX/lib
bash ./build.sh
```


Install addtional package
```bash
# install sky map
git clone https://github.com/reedessick/skymap_statistics.git
. setup.sh

# install baudline

# install aladin
```
<!-- Use llvm, not gcc -->
<!-- ```bash
export CC=gcc-11
export CXX=gcc-11
alias gcc=/usr/local/opt/llvm/bin/clang
alias g++=/usr/local/opt/llvm/bin/clang
``` -->

<!-- In Makefile, do not use version below c++17. Change to llvm (default clang doesn't support openmpi, gcc doesn't support ...)

There is a addition -I in tools/toolbox Makefile if not all options are provided

In file `tools/frdisplay/Makefile`, the `${HOME_FRLIB}/{UNAME}/libFrame.a` should be change to `${HOME_FRLIB}/lib/libFrame.a` and

```bash
export HOME_FRLIB="/usr/local/anaconda3/envs/cwb/lib"
```

Still have problem, skipping. -->